from collections import OrderedDict

from tmux_projector.utils import run_command
from tmux_projector.utils import debug_print
from tmux_projector.models.window import Window
from tmux_projector.models.options import OptionsManager


class Session:

    def __init__(self, session_name, options=None):
        self.session_name = session_name
        self.windows = []
        self.options_manager = OptionsManager(session_name, options)

    def add_option(self, option, value):
        self.options_manager.set_option(option, value)

    def create_window(self, window_json):
        window = Window.from_json(window_json, self.session_name)
        self.windows.append(window)
        return window

    def get_option(self, option):
        return self.options_manager.get_option(option)

    @staticmethod
    def from_json(session_json):
        session = Session(session_json['session_name'], session_json['options'])
        for window_json in session_json['windows']:
            window = session.create_window(window_json)
            for pane_json in window_json['panes']:
                pane = window.create_pane()
                pane._load_config_from_json(pane_json)
        return session

    def to_json(self):
        windows = [window.to_json() for window in self.windows]
        data = {}
        data['session_name'] = self.session_name
        data['windows'] = windows
        data['options'] = self.options_manager.to_json()
        return data

    def start(self):
        session_created = self._start_session()
        if session_created:
            self.options_manager.apply_options()
            for window_i, window in enumerate(self.windows):
                window.start(window_i + 1) # new-session already created a single window
            self._cleanup()

        if self.options_manager.get_option('auto_attach'):
            self._attach()
        else:
            print("Session is now ready. Attach to the session by running the command:")
            print(f"tmux attach -t {self.session_name}")

    def kill(self):
        kill_command = f'tmux kill-session -t {self.session_name}'
        run_command(kill_command)


    def _attach(self):
        attach_command = f'tmux attach -t {self.session_name}'
        run_command(attach_command)

    def _cleanup(self):
        ## Removing the first unnecessary window created when creating the session
        kill_window_command = f"tmux kill-window -t {self.session_name}:0"
        run_command(kill_window_command)
        renumber_windows_command = f"tmux move-window -t {self.session_name} -r"
        run_command(renumber_windows_command)


    def _start_session(self):
        start_command = f"tmux new -d -s {self.session_name}"
        run_command(start_command)
        return True






import math
import json
import random
from tmux_projector.utils import run_command
from tmux_projector.models.pane import Pane
from tmux_projector.exceptions import SliceException
from tmux_projector.models.options import OptionsManager

class Window:

    def __init__(self, window_name, session_name, options=None):
        self.window_name = window_name
        self.session_name = session_name
        self.panes = []
        self.options = {}
        self.options_manager = OptionsManager(session_name, options)

    @staticmethod
    def from_json(window_json, session_name):
        window = Window(window_json['window_name'], session_name)
        if 'layout' in window_json:
            window.set_option('layout', window_json['layout'])
        return window
        
    def get_option(self, option):
        return self.options_manager.get_option(option)

    def create_pane(self):
        pane = Pane(len(self.panes)+1, self.window_name, self.session_name)
        self.panes.append(pane)
        return pane

    def start(self, window_index):
        self._start_window(window_index)
        for pane_index, pane in enumerate(self.panes):
            pane.start()
        self._cleanup()
        self._set_layout()

    def set_option(self, option, value):
        self.options[option] = value

    def to_json(self):
        panes = [pane.to_json() for pane in self.panes]
        data = {'window_name': self.window_name, 'layout': self.get_option('layout'), 'panes': panes}
        return data 

    def _set_layout(self):
        if 'layout' in self.options:
            layout_command = f'tmux select-layout -t {self.session_name}:{self.window_name} {self.options["layout"]}'
            run_command(layout_command)

    def _start_window(self, window_index):
        create_window_command = f"tmux new-window -t {self.session_name}:{window_index} -n {self.window_name}"
        run_command(create_window_command)


    def _cleanup(self):
        ## Removing the first unnecessary pane created when creating this window
        kill_pane_command = f"tmux kill-pane -t {self.session_name}:{self.window_name}.0"
        run_command(kill_pane_command)







import sys
import math
import random
from tmux_projector.exceptions import ValidationException

ALLOWED_LAYOUTS = ['even-horizontal', 'even-vertical', 'main-horizontal', 'main-vertical', 'tiled']

class ConfigValidator:

    def __init__(self):
        pass

    def validate(self, session_json):
        try:
            self._validate_session(session_json)
            self._validate_windows(session_json)
            self._validate_panes(session_json)
            pass
        except ValidationException as e:
            print(e.message)
            sys.exit(1)

    def _validate_session(self, session_json):
        if not session_json.get('session_name', ''):
            raise ValidationException(f'Session requires a name')
        if session_json.get('layout', None):
            if session_json['layout'] not in ALLOWED_LAYOUTS:
                raise ValidationException(f'Layout {session_json["layout"]} not valid.')

    def _validate_windows(self, session_json):
        for window in session_json['windows']:
            pass

    def _validate_panes(self, session_json):
        for window in session_json['windows']:
            for pane in window['panes']:
                pass



import yaml
import io
import os
import json
from tmux_projector.utils import run_command
from tmux_projector.models.session import Session
from tmux_projector.validator import ConfigValidator


class TmuxProjector:

    def __init__(self):
        pass


    def initialize_project(self):
        """
        Creates a barebones project and saves it as a YAML file.
        """ 
        session = Session(session_name='test')
        session.add_option('auto_attach', True)
        window = session.create_window({'window_name': 'test-window', 'layout': None})
        pane = window.create_pane()
        pane.dir = os.getcwd()
        pane.cmd = 'echo test'


        self._save_config(session.to_json())

    def run(self, args):
        if os.path.exists('.tmux_projector.yaml'):
            session = self._load_config()
            running_sessions = self._get_running_sessions()
            if session.session_name in running_sessions:
                if args.restart:
                    session.kill()
                    session.start()
                else:
                    self._reconnect_to_session(session)
            else:
                session.start()
        else:
            print("No project config found. Initialize one by running 'tmux_projector init'")

    def kill(self):
        if os.path.exists('.tmux_projector.yaml'):
            session = self._load_config()
            running_sessions = self._get_running_sessions()
            if session.session_name in running_sessions:
                session.kill()


    def _load_config(self):
        session_json = yaml.load(open('.tmux_projector.yaml'), Loader=yaml.FullLoader)
        validator = ConfigValidator().validate(session_json)
        session = Session.from_json(session_json)
        return session 

    def _save_config(self, data):
        open('.tmux_projector.yaml', 'w').write(yaml.dump(data, sort_keys=False))

    def _get_running_sessions(self):
        tmux_ls_command = f'tmux ls'
        output = run_command(tmux_ls_command)
        sessions = set()
        for line in output.split("\n"):
            if not line.strip(): continue
            session_name = line.split(":")[0]
            sessions.add(session_name)
        return sessions


    def _reconnect_to_session(self, session):
        if session.get_option('auto_attach'):
            self._reattach(session)
        else:
            print(f"Session {session.session_name} is already running. Do you want to reattach to it?")
            decision = input('y/n: ')
            if decision.lower() == 'y':
                self._reattach(session)

    def _reattach(self, session):
        reattach_command = f'tmux attach -t {session.session_name}'
        run_command(reattach_command)

    



import argparse

from tmux_projector.projector import TmuxProjector


def main():
    parser = argparse.ArgumentParser(description='Tmux projector')
    subparser = parser.add_subparsers(help='Action to take', dest='action', required=True)

    init_parser = subparser.add_parser('init')

    start_parser = subparser.add_parser('start')
    start_parser.add_argument('--restart', action='store_true')

    kill_parser = subparser.add_parser('kill')

    args = parser.parse_args()

    projector = TmuxProjector()
    if args.action == 'init':
        projector.initialize_project()
    elif args.action == 'kill':
        projector.kill()
    else:
        projector.run(args)


if __name__ == "__main__":
    main()



import os
from tmux_projector.utils import run_command

class Pane:
    def __init__(self, pane_name, window_name, session_name):
        self.pane_name = pane_name
        self.window_name = window_name
        self.session_name = session_name
        self.dir = None
        self.cmd = None
        self.venv = None

    def _load_config_from_json(self, pane_json):
        self.dir = pane_json.get('dir', None)
        self.cmd = pane_json.get('cmd', None)
        self.venv = pane_json.get('venv', None)

    def start(self):
        self._start_pane()
        if self.venv:
            venv_path = os.path.join(self.venv, 'bin/activate')
            self._send_keys(f'source Space "{venv_path}"')
        if self.dir:
            self._send_keys(f'cd Space "{self._format_command(self.dir)}" Enter')

        self._clean()

        if self.cmd:
            self._send_keys(f'"{self._format_command(self.cmd)}" Enter')


    
    def to_json(self):
        data = {'pane_name': self.pane_name, 'dir': self.dir, 'cmd': self.cmd, 'venv': self.venv}
        return data

    def _clean(self):
        self._send_keys(f"clear")

    def _format_command(self, cmd):
        # cmd = cmd.replace(" ", " Space ")
        return cmd

    def _start_pane(self):
        create_pane_command = f"tmux split-window -t {self.session_name}:{self.window_name}"
        run_command(create_pane_command)


    def _send_keys(self, cmd):
        # command = self._format_command(cmd)
        run_command(f"tmux send-keys -t {self.session_name}:{self.window_name}.{self.pane_name} {cmd} Enter")






class ValidationException(Exception):

    def __init__(self, message='Generic validation exception'):
        self.message = message

    def __str__(self):
        return f'Validation failed! - {self.message}'


class SliceException(Exception):

    def __init__(self, message='Generic slice exception'):
        self.message = message

    def __str__(self):
        return f'Slice failed! - {self.message}'



import subprocess

debug = False

def run_command(command):
    # print("Running cmd:", command)
    if debug:
        print(f"Running command {command}")
    result = subprocess.run(command, capture_output=True, shell=True)
    return result.stdout.decode()


def debug_print(output):
    # print(output)
    pass



import setuptools


setuptools.setup(
    name="tmux-projector",
    version="0.0.2",
    author_email="avjves@gmail.com",
    description="Tool to start tmux projects more easily",
    packages=setuptools.find_packages(),
    python_requires=">=3.7",
    entry_points={
        'console_scripts': ['ts=tmux_projector.run:main']
    }
)



from tmux_projector.utils import run_command

class OptionsManager:

    def __init__(self, session_name, options):
        self.session_name = session_name
        self.options = options if options else {}

    def apply_options(self):
        self._apply_mouse()

    def set_option(self, option, value):
        self.options[option] = value

    def get_option(self, option):
        return self.options.get(option, None)

    def _apply_mouse(self):
        mouse_enabled = self.get_option('mouse')
        if mouse_enabled != None:
            value = 'on' if mouse_enabled else 'off'
            mouse_command = f'tmux set -g mouse {value}'
            run_command(mouse_command)

    def to_json(self):
        return self.options
